/*
 * infos.cpp
 *
 * classes feature_info, enum_const_info, segment_info
 *
 * Ulrik Petersen
 * Created: 2/26-2001
 * Last update: 11/27-2002
 *
 */
/************************************************************************
 *
 *   Emdros - the database engine for analyzed or annotated text
 *   Copyright (C) 2001-2002  Ulrik Petersen
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, license version 2.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *
 *   Other licensing forms
 *   =====================
 *
 *   If you wish to negotiate commercial licensing, please contact
 *   Ulrik Petersen at ulrikp[at]users.sourceforge.net.
 *
 *   Licensing can also be negotiated if your organization is an
 *   educational, non-profit, charity, missionary or similar
 *   organization.
 *
 **************************************************************************/


#include <infos.h>


///////////////////////////////////////////////////////////
//
// feature_info
//
///////////////////////////////////////////////////////////

feature_info::feature_info(std::string name, id_d_t type,
			   std::string default_value, bool is_computed) : 
   m_name(name), 
   m_type(type), 
   m_default_value(default_value), 
   m_is_computed(is_computed)
{
}

feature_info::feature_info(const feature_info& other)
{
  copy_other(other);
}

feature_info::~feature_info()
{
}

feature_info& feature_info::operator=(const feature_info& other)
{
  copy_other(other);
  return *this;
}

bool feature_info::operator==(const feature_info& other) const
{
  if (m_name == other.m_name
      && m_type == other.m_type
      && m_default_value == other.m_default_value
      && m_is_computed == other.m_is_computed)
    return true;
  else
    return false;
}

void feature_info::copy_other(const feature_info& other)
{
  m_name = other.m_name; 
  m_type = other.m_type; 
  m_default_value = other.m_default_value; 
  m_is_computed = other.m_is_computed;
}



const std::string& feature_info::get_name(void) const
{
  return m_name;
}

id_d_t feature_info::get_type(void) const
{
  return m_type;
}

const std::string& feature_info::get_default_value(void) const
{
  return m_default_value;
}

bool feature_info::get_is_computed(void) const
{
  return m_is_computed;
}




///////////////////////////////////////////////////////////
//
// enum_const_info
//
///////////////////////////////////////////////////////////


enum_const_info::enum_const_info(std::string name, int value, bool is_default):
  m_name(name), 
  m_value(value), 
  m_is_default(is_default)
{

}

enum_const_info::~enum_const_info()
{

}

const std::string& enum_const_info::get_name(void) const
{
  return m_name;
}

int enum_const_info::get_value(void) const
{
  return m_value;
}

bool enum_const_info::get_is_default(void) const
{
  return m_is_default;
}

bool enum_const_info::operator==(const enum_const_info& other) const
{
  if (m_name == other.m_name
      && m_value == other.m_value
      && m_is_default == other.m_is_default)
    return true;
  else
    return false;
}




///////////////////////////////////////////////////////////
//
// segment_info
//
///////////////////////////////////////////////////////////


segment_info::segment_info(std::string name, monad_m first, monad_m last) : 
  m_name(name), 
  m_first(first), 
  m_last(last)
{
}

segment_info::~segment_info()
{
}

const std::string& segment_info::get_name(void) const
{
  return m_name;
}

monad_m segment_info::get_first(void) const
{
  return m_first;
}

monad_m segment_info::get_last(void) const
{
  return m_last;
}

bool segment_info::operator==(const segment_info& other) const
{
  if (m_name == other.m_name
      && m_first == other.m_first
      && m_last == other.m_last)
    return true;
  else
    return false;
}





