/*
 * mysqlemdfdb.cpp
 *
 * MySQL EMdF database base class
 *
 * Ulrik Petersen
 * Created: 1/27-2001
 * Last update: 5/29-2004
 *
 */
/************************************************************************
 *
 *   Emdros - the database engine for analyzed or annotated text
 *   Copyright (C) 2001-2004  Ulrik Petersen
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, license version 2.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *
 *   Other licensing forms
 *   =====================
 *
 *   If you wish to negotiate commercial licensing, please contact
 *   Ulrik Petersen at ulrikp[at]users.sourceforge.net.
 *
 *   Licensing can also be negotiated if your organization is an
 *   educational, non-profit, charity, missionary or similar
 *   organization.
 *
 **************************************************************************/


#ifdef _WIN32
#include <windows.h>
#endif

#include <mysqlemdfdb.h>
#include <mysqlconn.h>
#include <utils.h>


MySQLEMdFDB::MySQLEMdFDB(std::string host, std::string user, std::string passwd, std::string database_name) 
{
  pConn = new MySQLEMdFConnection(host, user, passwd, database_name);
}


MySQLEMdFDB::~MySQLEMdFDB()
{
}

bool MySQLEMdFDB::UseDatabase(const std::string& db_name)
{
  if (pConn == 0)
    return false;
  else {
    // Convert name
    std::string NewDbName;
    ConvertDBName(db_name, NewDbName);
    
    // Ask the connection to do it.
    bool pConnResult = pConn->UseDatabase(NewDbName);

    // Call base class
    EMdFDB::UseDatabase(db_name);

    return pConnResult;
  }
}

bool MySQLEMdFDB::SetNextObjectID_DIfNotHigher(id_d_t next_id_d)
{
  if (pConn == 0)
    return false;
  else {
    // Declare szQuery
    char szQuery[512];

    // Begin transaction
    bool bDoCommit = pConn->BeginTransaction();

    try {
      // Ask what the current value is
      id_d_t current_id_d;
      sprintf(szQuery, "SELECT sequence_value FROM sequence_118_%d;", SEQUENCE_OBJECT_ID_DS);
      if (!pConn->Exec(szQuery)) {
	DEBUG_QUERY_FAILED("MySQLEMdFDB::SetNextObjectID_DIfNotHigher", szQuery);
	if (bDoCommit)
	  pConn->AbortTransaction();
	return false;
      }

      // Access result
      if (!pConn->AccessTuple(0, 0, "MySQLEMdFDB::SetNextObjectID_DIfNotHigher", current_id_d)) {
	// DEBUG information has already been printed if it is on.
	if (bDoCommit)
	  pConn->AbortTransaction();
	return false;
      }

      // Only act if current value is less than next minus 1
      if (current_id_d < next_id_d-1) {
	// Update value
	sprintf(szQuery, 
		"UPDATE sequence_118_%d\n"
		"SET sequence_value = LAST_INSERT_ID(%ld);",
		SEQUENCE_OBJECT_ID_DS, next_id_d-1);
	if (!pConn->Exec(szQuery)) {
	  DEBUG_QUERY_FAILED("MySQLEMdFDB::SetNextObjectID_DIfNotHigher", szQuery);
	  if (bDoCommit)
	    pConn->AbortTransaction();
	  return false;
	}
      }

      // Commit transaction
      if (bDoCommit) {
	if (!pConn->CommitTransaction()) {
	  DEBUG_COMMITTRANSACTION_FAILED("MySQLEMdFDB::SetNextObjectID_DIfNotHigher");
	  return false;
	}
      }
    } catch (EMdFNULLValueException e) {
      // Abort transaction if in progress
      if (bDoCommit)
	pConn->AbortTransaction();

      DEBUG_NULL_VALUE_EXCEPTION("MySQLEMdFDB::SetNextObjectID_DIfNotHigher");

      // There was a NULL value exception, hence we return an error
      return false;
    }

    // If we got this far, the command succeeded.
    return true;
  }
}
