/*
 * pgemdfdb.cpp
 *
 * PostgreSQL EMdF database base class
 *
 * Ulrik Petersen
 * Created: 1/27-2001
 * Last update: 5/19-2004
 *
 */
/************************************************************************
 *
 *   Emdros - the database engine for analyzed or annotated text
 *   Copyright (C) 2001-2004  Ulrik Petersen
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, license version 2.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *
 *   Other licensing forms
 *   =====================
 *
 *   If you wish to negotiate commercial licensing, please contact
 *   Ulrik Petersen at ulrikp[at]users.sourceforge.net.
 *
 *   Licensing can also be negotiated if your organization is an
 *   educational, non-profit, charity, missionary or similar
 *   organization.
 *
 **************************************************************************/


#include <pgemdfdb.h>
#include <pgconn.h>
#include <encryption.h>
#include <utils.h>

PgEMdFDB::PgEMdFDB(std::string host, std::string user, std::string passwd, std::string database_name) : m_host(host), m_user(user), m_passwd(passwd)
{
  pConn = new PgEMdFConnection(host, user, passwd, database_name);
  encrypt_string(m_passwd);
}


PgEMdFDB::~PgEMdFDB()
{
}

bool PgEMdFDB::UseDatabase(const std::string& db_name)
{
  if (pConn == 0)
    return false;
  else {
    std::string NewDbName;
    ConvertDBName(db_name, NewDbName);

    std::string unencrypted_password = m_passwd;
    unencrypt_string(unencrypted_password);

    delete pConn;
    pConn = new PgEMdFConnection(m_host, m_user, unencrypted_password, NewDbName);

    zero_fill_string(unencrypted_password);

    // Call base class
    EMdFDB::UseDatabase(db_name);

    return pConn->ConnectionOk();
  }
}

bool PgEMdFDB::SetNextObjectID_DIfNotHigher(id_d_t next_id_d)
{
  if (pConn == 0)
    return false;
  else {
    // Declare szQuery
    char szQuery[512];

    // Begin transaction
    bool bDoCommit = pConn->BeginTransaction();

    try {
      // Ask what the current value is
      id_d_t current_id_d;
      sprintf(szQuery, "SELECT currval('sequence_118_%d');", SEQUENCE_OBJECT_ID_DS);
      if (!pConn->Exec(szQuery)) {
	DEBUG_QUERY_FAILED("PgEMdFDB::SetNextObjectID_DIfNotHigher", szQuery);
	if (bDoCommit)
	  pConn->AbortTransaction();
	return false;
      }

      // Access result
      if (!pConn->AccessTuple(0, 0, "PgEMdFDB::SetNextObjectID_DIfNotHigher", current_id_d)) {
	// DEBUG information has already been printed if it is on.
	if (bDoCommit)
	  pConn->AbortTransaction();
	return false;
      }

      // Clean up
      pConn->ReleaseResults();

      // Only act if current value is less than next minus 1
      if (current_id_d < next_id_d-1) {
	// Set current value to one less than next_id_d, so that
	// next time, we will increment to next_id_d.
	sprintf(szQuery, "SELECT setval('sequence_118_%d', %ld);", SEQUENCE_OBJECT_ID_DS, next_id_d-1);
	if (!pConn->Exec(szQuery)) {
	  DEBUG_QUERY_FAILED("PgEMdFDB::SetNextObjectID_DIfNotHigher", szQuery);
	  if (bDoCommit)
	    pConn->AbortTransaction();
	  return false;
	}

	// Clean up
	pConn->ReleaseResults();
      }


      // Commit transaction
      if (bDoCommit) {
	if (!pConn->CommitTransaction()) {
	  DEBUG_COMMITTRANSACTION_FAILED("PgEMdFDB::SetNextObjectID_DIfNotHigher");
	  return false;
	}
      }
    } catch (EMdFNULLValueException e) {
      // Abort transaction if in progress
      if (bDoCommit)
	pConn->AbortTransaction();

      DEBUG_NULL_VALUE_EXCEPTION("PgEMdFDB::SetNextObjectID_DIfNotHigher");

      // There was a NULL value exception, hence we return an error
      return false;
    }

    // If we got this far, the command succeeded.
    return true;
  }
}

