/*
 * string_func.cpp
 *
 * String helper functions
 *
 * Ulrik Petersen
 * Created: 3/1-2001
 * Last update: 3/2-2004
 *
 */
/************************************************************************
 *
 *   Emdros - the database engine for analyzed or annotated text
 *   Copyright (C) 2001-2004  Ulrik Petersen
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, license version 2.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *
 *   Other licensing forms
 *   =====================
 *
 *   If you wish to negotiate commercial licensing, please contact
 *   Ulrik Petersen at ulrikp[at]users.sourceforge.net.
 *
 *   Licensing can also be negotiated if your organization is an
 *   educational, non-profit, charity, missionary or similar
 *   organization.
 *
 **************************************************************************/


#include <string_func.h>
#include <cstdio>
#include <cctype>
#include <cstring>


void str_toupper(const std::string& in, std::string& result)
{
  result = "";
  std::string::const_iterator ci(in.begin());
  std::string::const_iterator cend(in.end());
  while (ci != cend) {
    result += (char) (toupper(*ci));
    ++ci;
  }
}

void str_tolower(const std::string& in, std::string& result)
{
  result = "";
  std::string::const_iterator ci(in.begin());
  std::string::const_iterator cend(in.end());
  while (ci != cend) {
    result += (char) (tolower(*ci));
    ++ci;
  }
}

// See Bjarne Stroustrup, 
// The C++ Programming Language,
// 3rd Edition, 1997
// p. 591
int strcmp_nocase(const std::string& str1, const std::string& str2)
{
  std::string::const_iterator p = str1.begin();
  std::string::const_iterator str1end = str1.end();
  std::string::const_iterator q = str2.begin();
  std::string::const_iterator str2end = str2.end();
  while (p != str1end && q != str2end) {
    char tolop = tolower(*p);
    char toloq = tolower(*q);
    if (tolop != toloq)
      return (tolop < toloq) ? -1 : 1;
    ++p;
    ++q;
  }
  std::string::size_type str1size = str1.size();
  std::string::size_type str2size = str2.size();
  return (str2size == str1size) ? 0 : (str1size < str2size) ? -1 : 1;
}

// See Bjarne Stroustrup, 
// The C++ Programming Language,
// 3rd Edition, 1997
// p. 591
int strcmp_case(const std::string& str1, const std::string& str2)
{
  std::string::const_iterator p = str1.begin();
  std::string::const_iterator str1end = str1.end();
  std::string::const_iterator q = str2.begin();
  std::string::const_iterator str2end = str2.end();
  while (p != str1end && q != str2end) {
    char cp = *p;
    char cq = *q;
    if (cp != cq)
      return (cp < cq) ? -1 : 1;
    ++p;
    ++q;
  }
  std::string::size_type str1size = str1.size();
  std::string::size_type str2size = str2.size();
  return (str2size == str1size) ? 0 : (str1size < str2size) ? -1 : 1;
}

std::string bool2string(bool b)
{
  if (b)
    return "1";
  else
    return "0";
}

bool string2bool(const std::string& str)
{
  return str == "1";
}

std::string bool_alpha2string(bool b)
{
  if (b)
    return "true";
  else
    return "false";
}

bool string2bool_alpha(const std::string& str)
{
  return str == "true";
}

std::string int2string(int i)
{
  char szResult[30];
  sprintf(szResult, "%d", i);
  return szResult;
}

std::string monad_m2string(monad_m m)
{
  char szResult[30];
  sprintf(szResult, "%ld", (long) m);
  return szResult;
}

monad_m string2monad_m(const std::string& str)
{
  return strtol(str.c_str(), (char **)NULL, 10);
}

std::string id_d2string(id_d_t i)
{
  char szResult[30];
  if (i == NIL) {
    strcpy(szResult, NIL_AS_VISIBLE_STRING);
  } else {
    sprintf(szResult, "%ld", (long) i);
  }
  return szResult;
}

std::string long2string(long l)
{
  char szResult[30];
  sprintf(szResult, "%ld", l);
  return szResult;
}

long string2long(const std::string& str)
{
  return strtol(str.c_str(), (char **)NULL, 10);
}

id_d_t string2id_d(const std::string& str)
{
  id_d_t id_d;
  if (str == NIL_AS_VISIBLE_STRING) {
    id_d = NIL;
  } else {
    id_d = strtol(str.c_str(), (char **)NULL, 10);
  }
  return id_d;
}

inline std::string char2hex(char c)
{
  char szResult[5];
  sprintf(szResult, "%02x", c);
  return szResult;
}


std::string string2dbstring(const std::string& str)
{
  std::string result;
  for (std::string::size_type i = 0; i < str.length(); ++i) {
    result += char2hex(str[i]);
  }
  return result;
}

inline char hex2char(char a, char b)
{
  char szTemp[5];
  sprintf(szTemp, "0x%c%c", a, b);
  unsigned int c;
  sscanf(szTemp, "%x", &c);
  return (char) c;
}

std::string dbstring2string(const std::string& str)
{
  std::string result;
  for (std::string::size_type i = 0; i < str.length(); i += 2) {
    result += hex2char(str[i], str[i+1]);
  }
  return result;
}

// Encode a string so it can be included in "double quotes" in an MQL script
std::string encodeSTRINGstring(const std::string& str)
{
  std::string result;
  for (std::string::size_type i = 0; i < str.length(); ++i) {
    unsigned char c = (unsigned char) str[i];
    if (c < 0x20 
	|| c >= 0x80) {
      result += "\\x" + char2hex(c);
    } else {
      switch (c) {
      case '\\':
	result += "\\\\";
	break;
      case '\"':
	result += "\\\"";
	break;
      default:
	result += c;
	break;
      }
    }
  }
  return result;
}

std::string encodeASCIIstring(const std::string& str)
{
  std::string result;
  for (std::string::size_type i = 0; i < str.length(); ++i) {
    switch (str[i]) {
    case '\\':
      result += "\\\\";
      break;
    case '\'':
      result += "''";
      break;
    default:
      result += str[i];
      break;
    }
  }
  return result;
}

bool is_other_than_whitespace(const std::string& str)
{
  return str.find_first_not_of(std::string(" \t\n\r\v"), 0) != std::string::npos;
}

bool has_double_quote(const std::string& str)
{
  return str.find_first_of('"') != std::string::npos;
}

