/*
 * mql_monads_statements.cpp
 *
 * MQL MONADS statements
 *
 * Ulrik Petersen
 * Created: 3/6-2001 (March 6, 2001)
 * Last update: 3/26-2003
 *
 */
/************************************************************************
 *
 *   Emdros - the database engine for analyzed or annotated text
 *   Copyright (C) 2001-2003  Ulrik Petersen
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, license version 2.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *
 *   Other licensing forms
 *   =====================
 *
 *   If you wish to negotiate commercial licensing, please contact
 *   Ulrik Petersen at ulrikp[at]users.sourceforge.net.
 *
 *   Licensing can also be negotiated if your organization is an
 *   educational, non-profit, charity, missionary or similar
 *   organization.
 *
 **************************************************************************/


#include <mql_monads_statements.h>
#include <string_func.h>

/////////////////////////////////////////////////////////////
//
// CMonadsStatement
//
// base class
//
////////////////////////////////////////////////////////////
CMonadsStatement::~CMonadsStatement()
{
  delete m_AST_monads;
}

void CMonadsStatement::Weed(bool& bResult)
{
  // TODO: Implement
  bResult = false;
  m_pEE->pError->mql_append_error("Not implemented yet.\n");
  return;
}

bool CMonadsStatement::Symbol(bool& bResult)
{
  // TODO: Implement
  m_pEE->pError->mql_append_error("Not implemented yet.\n");
  return false;
}



/////////////////////////////////////////////////////////////
//
// INSERT MONADS
//
// CInsertMonadsStatement
//
////////////////////////////////////////////////////////////
void CInsertMonadsStatement::Weed(bool& bResult)
{
  // TODO: Implement
  bResult = false;
  m_pEE->pError->mql_append_error("Not implemented yet.\n");
  return;
}

bool CInsertMonadsStatement::Symbol(bool& bResult)
{
  // TODO: Implement
  m_pEE->pError->mql_append_error("Not implemented yet.\n");
  return false;
}

bool CInsertMonadsStatement::Exec()
{
  // TODO: Implement
  m_pEE->pError->mql_append_error("Not implemented yet.\n");
  return false;
}



/////////////////////////////////////////////////////////////
//
// DELETE MONADS
//
// CDeleteMonadsStatement
//
////////////////////////////////////////////////////////////
void CDeleteMonadsStatement::Weed(bool& bResult)
{
  // TODO: Implement
  bResult = false;
  m_pEE->pError->mql_append_error("Not implemented yet.\n");
  return;
}

bool CDeleteMonadsStatement::Symbol(bool& bResult)
{
  // TODO: Implement
  m_pEE->pError->mql_append_error("Not implemented yet.\n");
  return false;
}

bool CDeleteMonadsStatement::Exec()
{
  // TODO: Implement
  m_pEE->pError->mql_append_error("Not implemented yet.\n");
  return false;
}


/////////////////////////////////////////////////////////////
//
// GET MONADS
//
// CGetMonadsStatement
//
////////////////////////////////////////////////////////////
CGetMonadsStatement::CGetMonadsStatement(CMQL_execution_environment *pEE, 
		    CID_D *pID_Ds, 
		    std::string* object_type_name)
  : CStatement(pEE), m_id_ds(pID_Ds), m_object_type_name(object_type_name),
    m_object_type_id(NIL)
{
}

CGetMonadsStatement::~CGetMonadsStatement()
{
  delete m_id_ds;
  delete m_object_type_name;
}

bool CGetMonadsStatement::Exec()
{
  // Make list
  std::list<id_d_t> id_ds;
  m_id_ds->ExecMakeList(id_ds);

  // Make table
  m_result = new CMQLResult();

  // Build header
  m_result->append_header("object_id_d", kTCID_D);
  m_result->append_header("mse_first", kTCMonad_m);
  m_result->append_header("mse_last", kTCMonad_m);

  // Ask EMdFDB
  if (!m_pEE->pDB->GetMonadsFromID_Ds(id_ds, *m_object_type_name, m_result)) {
    // Failure
    return false;
  }

  // If we got this far, there was nothing wrong.
  return true;
}


/////////////////////////////////////////////////////////////
//
// CMonadSetCalculationStatement
//
////////////////////////////////////////////////////////////

CMonadSetCalculationStatement::CMonadSetCalculationStatement(CMQL_execution_environment *pEE, CMonadSetChainElement *chain) : CStatement(pEE), m_chain(chain) 
{
}

CMonadSetCalculationStatement::~CMonadSetCalculationStatement()
{
  delete m_chain;
}

void CMonadSetCalculationStatement::Weed(bool& bResult)
{
  // Weed monotonic
  m_chain->WeedMonotonic(m_pEE, bResult);
  if (!bResult)
    return;

  // Weed positive
  m_chain->WeedPositive(m_pEE, bResult);
  if (!bResult)
    return;

  return;
}

bool CMonadSetCalculationStatement::Exec()
{
  // Calculate result
  CSet_of_monad_ms monads;
  m_chain->Exec(monads);

  // Make table
  m_result = new CMQLResult();

  // Build header
  m_result->append_header("mse_first", kTCMonad_m);
  m_result->append_header("mse_last", kTCMonad_m);

  // Make table
  SOMConstIterator ci = monads.const_iterator();
  while (ci.hasNext()) {
    Monad_Set_Element mse = ci.next();
    m_result->start_new_row();
    m_result->append(monad_m2string(mse.first()));
    m_result->append(monad_m2string(mse.last()));
  }

  // There can be no DB errors
  return true;
}


