/*
 * mql_result.cpp
 *
 * MQL results
 *
 * Ulrik Petersen
 * Created: 3/8-2001 (8th of March, 2001)
 * Last update: 3/22-2003
 *
 */
/************************************************************************
 *
 *   Emdros - the database engine for analyzed or annotated text
 *   Copyright (C) 2001-2003  Ulrik Petersen
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, license version 2.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *
 *   Other licensing forms
 *   =====================
 *
 *   If you wish to negotiate commercial licensing, please contact
 *   Ulrik Petersen at ulrikp[at]users.sourceforge.net.
 *
 *   Licensing can also be negotiated if your organization is an
 *   educational, non-profit, charity, missionary or similar
 *   organization.
 *
 **************************************************************************/


#include <mql_result.h>
#include <mql_execution_environment.h>
#include <mql_sheaf.h>
#include <emdf_output.h>
#include <assert.h>

// For lists
CMQLResult::CMQLResult() 
  : m_sheaf(0), m_bDeleteResult(true)
{
}

// For sheafs
CMQLResult::CMQLResult(CSheaf* sheaf) 
  : m_sheaf(sheaf), m_bDeleteResult(true)
{
}

// Used by other methods to take over result
void CMQLResult::set_delete_result(bool bDoDelete)
{
  m_bDeleteResult = bDoDelete;
}

CMQLResult::~CMQLResult()
{
  if (m_bDeleteResult)
    delete m_sheaf;
}

bool CMQLResult::isSheaf() const
{
  return m_sheaf != 0;
}

bool CMQLResult::isTable() const
{
  return !isSheaf();
}

void CMQLResult::Print(CEMdFOutput *pOut, CMQL_execution_environment *pEE) const
{
  switch (pEE->pOut->get_output_kind()) {
  case kOKXML:
    if (m_sheaf != 0) {
      m_sheaf->put_XML(pOut, pEE);
    } else {
      CTable::PrintXML(pOut);
    }
    break;
  case kOKConsole:
    if (m_sheaf != 0) {
      m_sheaf->put_Console(pOut, pEE);
    } else {
      CTable::PrintConsole(pOut);
    }
    break;
  default:
    assert(false);
    break;
  }
}

void CMQLResult::print_DTD(CEMdFOutput* pOut)
{
  pOut->out("  <!ELEMENT mql_results (mql_result)* >");
  pOut->newline();
  pOut->out("  <!ELEMENT mql_result ((mql_progress)?, status, (error)?, (warning)?, (table|sheaf)?) >");
  pOut->newline();
  pOut->out("  <!ELEMENT mql_progress (#PCDATA)* >");
  pOut->newline();
  pOut->out("  <!ELEMENT status EMPTY >");
  pOut->newline();
  pOut->out("  <!ELEMENT error (error_source, error_stage, ");
  pOut->newline();
  pOut->out("                   error_message_db, error_message_compiler) >");
  pOut->newline();
  pOut->out("  <!ELEMENT error_source EMPTY >");
  pOut->newline();
  pOut->out("  <!ELEMENT error_stage EMPTY >");
  pOut->newline();
  pOut->out("  <!ELEMENT error_message_db (#PCDATA)* >");
  pOut->newline();
  pOut->out("  <!ELEMENT error_message_compiler (#PCDATA)* >");
  pOut->newline();
  pOut->out("  <!ELEMENT warning (#PCDATA)* >");
  pOut->newline();
  pOut->out("  <!ATTLIST mql_results >");
  pOut->newline();
  pOut->out("  <!ATTLIST mql_result  >");
  pOut->newline();
  pOut->out("  <!ATTLIST status ");
  pOut->newline();
  pOut->out("                success (true|false) #REQUIRED >");
  pOut->newline();
  pOut->out("  <!ATTLIST error >");
  pOut->newline();
  pOut->out("  <!ATTLIST error_source ");
  pOut->newline();
  pOut->out("                source  (compiler|db) #REQUIRED >");
  pOut->newline();
  pOut->out("  <!ATTLIST error_stage  ");
  pOut->newline();
  pOut->out("                stage   (none|parse|weed|symbol|type|monads|exec) #REQUIRED >");
  pOut->newline();
  pOut->out("  <!ATTLIST error_message_db >");
  pOut->newline();
  pOut->out("  <!ATTLIST error_message_compiler >");
  pOut->newline();
  pOut->out("  <!ATTLIST warning >");
  pOut->newline();
}

