/*
 * libemdfperl.i: SWIG interface to libemdf.la from Perl
 *   by Chris Wilson <emdros@qwirx.com>
 *   Created: Fri Dec 20 2002
 *   Last update: 3/18-2003
 *
 *   stolen from:
 * libemdfruby.i: SWIG interface to libemdf.la
 *   by Kirk Lowery <klowery@wts.edu>
 *   Mon Oct 28 10:09:52 EST 2002
 *
 */
/************************************************************************
 *
 *   Emdros - the database engine for analyzed or annotated text
 *   Copyright (C) 2002-2003  Ulrik Petersen
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, license version 2.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *
 *   Other licensing forms
 *   =====================
 *
 *   If you wish to negotiate commercial licensing, please contact
 *   Ulrik Petersen at ulrikp[at]users.sourceforge.net.
 *
 *   Licensing can also be negotiated if your organization is an
 *   educational, non-profit, charity, missionary or similar
 *   organization.
 *
 **************************************************************************/

%{
#undef list
#include "../../include/emdf_output.h"
#include "../../include/emdfdb.h"
#include "../../include/monads.h"
#include "../../include/table.h"
%}

%include "../../include/emdf_output.h"
%include "../../include/emdfdb.h"
%include "../../include/monads.h"
%include "../../include/table.h"



%inline %{
std::string CTable_get_row_col(CTable *table, int dest_row, int dest_col) {
	TableIterator row_iter = table->iterator();
	int row_num;
	for (row_num = 0; row_num < dest_row && row_iter.hasNext();
	     ++row_num, row_iter.next());
	if (row_num != dest_row) {
		std::cerr << "No such row " << dest_row << "\n";
		return NULL;
	}
	TableRowIterator col_iter = row_iter.current().iterator();
	int col_num;
	for (col_num = 0; col_num < dest_col && col_iter.hasNext();
	     ++col_num, col_iter.next());
	if (col_num != dest_col) {
		std::cerr << "No such column " << dest_col << "\n";
		return NULL;
	}
	return col_iter.current();
}
%}

/* convert a CTable to a Perl array */

/* References:
 * Array handling example: http://www.swig.org/Doc1.3/Perl5.html#n33
 * XS arrays reference: man perlguts or
 *   http://www.perldoc.com/perl5.6/pod/perlguts.html#Working-with-AVs
 */

%typemap(out) CTable * {
	TableIterator this_row_iter = $1->iterator();
	int total_num_rows = $1->size();
	SV** row_svs = new SV* [total_num_rows];

	for (int this_row_num = 0; this_row_num < total_num_rows;
		++this_row_num) {
		TableRowIterator this_col_iter = this_row_iter.current().iterator();
		int total_num_cols = this_row_iter.current().size();
		SV** col_svs = new SV* [total_num_cols];

		for (int this_col_num = 0; this_col_num < total_num_cols;
			++this_col_num) {
			const char* this_string = 
				this_col_iter.current().c_str();
			// strdup(this_col_iter->c_str());
			SV* this_sv = sv_newmortal();
			sv_setpv(this_sv, this_string);
			// FIXME this is not binary safe - need length param
			col_svs[this_col_num] = this_sv;
			this_col_iter.next();
		}

		AV* this_row_av = av_make(total_num_cols, col_svs);
		delete col_svs;

		SV* this_row_rv = newRV((SV*)this_row_av);
		row_svs[this_row_num] = this_row_rv;
		this_row_iter.next();
	}

	AV* all_rows_av = av_make(total_num_rows, row_svs);
	delete row_svs;

	$result = newRV((SV*)all_rows_av);
	sv_2mortal($result);
	++argvi;
}

// this function does nothing much, but the above typemap applies to it
// and makes it work.

%inline %{
CTable * CTable_to_array (CTable * table) {
	return table;
}
%}

