/*
 * debug.h
 *
 * Debug defines for EMdF
 *
 * Ulrik Petersen
 * Created: 1/30-2001
 * Last update: 6/9-2003
 *
 */
/************************************************************************
 *
 *   Emdros - the database engine for analyzed or annotated text
 *   Copyright (C) 2001-2002  Ulrik Petersen
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, license version 2.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *
 *   Other licensing forms
 *   =====================
 *
 *   If you wish to negotiate commercial licensing, please contact
 *   Ulrik Petersen at ulrikp[at]users.sourceforge.net.
 *
 *   Licensing can also be negotiated if your organization is an
 *   educational, non-profit, charity, missionary or similar
 *   organization.
 *
 **************************************************************************/




#ifndef DEBUG__H__
#define DEBUG__H__

#include <iostream>
#include <sstream>

#ifndef EMDF_DEBUG
#define DEBUG_PRINT_ERROR_MESSAGE(err_msg)
#else // EMDF_DEBUG
#define DEBUG_PRINT_ERROR_MESSAGE(err_msg) std::cerr << err_msg;
#endif // EMDF_DEBUG

#define DEBUG_HANDLE_ERROR_MESSAGE(szError) {\
    std::string err_msg = szError; \
    append_local_error(err_msg); \
    DEBUG_PRINT_ERROR_MESSAGE(err_msg); \
  }

#define DEBUG_QUERY_FAILED(szMethod, szQ) { \
    std::ostringstream strerr_msg; \
    strerr_msg << szMethod << ": Query '" << std::endl \
         << szQ << std::endl \
         << "' failed." << std::endl; \
    DEBUG_HANDLE_ERROR_MESSAGE(strerr_msg.str()); \
}
#define DEBUG_X_FAILED(szMethod, szWhatFailed) { \
    std::ostringstream strerr_msg; \
    strerr_msg << szMethod << ": " << szWhatFailed << " failed." << std::endl;\
    DEBUG_HANDLE_ERROR_MESSAGE(strerr_msg.str()); \
}
#define DEBUG_NULL_VALUE_EXCEPTION(szMethod) { \
    std::ostringstream strerr_msg; \
    strerr_msg << szMethod << std::endl; \
    DEBUG_HANDLE_ERROR_MESSAGE(strerr_msg.str()); \
}
#define DEBUG_BEGINTRANSACTION_FAILED(szMethod) DEBUG_X_FAILED(szMethod, "BeginTransaction")
#define DEBUG_COMMITTRANSACTION_FAILED(szMethod) DEBUG_X_FAILED(szMethod, "CommitTransaction")
#define DEBUG_ACCESS_TUPLE_FAILED(szMethod, row, field) { \
    std::ostringstream strerr_msg; \
    strerr_msg << szMethod << ": AccessTuple(row: " << row << ", field: " << field << ") failed." << std::endl; \
    DEBUG_HANDLE_ERROR_MESSAGE(strerr_msg.str()); \
}



#endif /* DEBUG__H__ */
