/*
 * infos.h
 *
 * classes feature_info, enum_const_info, segment_info
 *
 * Ulrik Petersen
 * Created: 2/26-2001
 * Last update: 11/27-2002
 *
 */
/************************************************************************
 *
 *   Emdros - the database engine for analyzed or annotated text
 *   Copyright (C) 2001-2002  Ulrik Petersen
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, license version 2.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *
 *   Other licensing forms
 *   =====================
 *
 *   If you wish to negotiate commercial licensing, please contact
 *   Ulrik Petersen at ulrikp[at]users.sourceforge.net.
 *
 *   Licensing can also be negotiated if your organization is an
 *   educational, non-profit, charity, missionary or similar
 *   organization.
 *
 **************************************************************************/



#ifndef INFOS__H__
#define INFOS__H__

#include <emdf.h>
#include <list>
#include <string>

class feature_info {
private:
  std::string m_name;
  id_d_t m_type;
  std::string m_default_value;
  bool m_is_computed;
public:
  feature_info(std::string name, id_d_t type,
	       std::string default_value, bool is_computed);
  feature_info(const feature_info& other);
  feature_info() {};
  feature_info& operator=(const feature_info& other);
  bool operator==(const feature_info& other) const;
  void copy_other(const feature_info& other);
  ~feature_info();
  const std::string& get_name(void) const;
  id_d_t get_type(void) const;
  const std::string& get_default_value(void) const;
  bool get_is_computed(void) const;
};



class enum_const_info {
private:
  std::string m_name;
  int m_value;
  bool m_is_default;
public:
  enum_const_info(std::string name, int value, bool is_default);
  ~enum_const_info();
  const std::string& get_name(void) const;
  int get_value(void) const;
  bool get_is_default(void) const;
  bool operator==(const enum_const_info& other) const;
};




class segment_info {
private:
  std::string m_name;
  monad_m m_first, m_last;
public:
  segment_info(std::string name, monad_m first, monad_m last);
  ~segment_info();
  const std::string& get_name(void) const;
  monad_m get_first(void) const;
  monad_m get_last(void) const;
  bool operator==(const segment_info& other) const;
};




#endif // INFOS__H__
