/*
 * mql_monads_statements.h
 *
 * Definitions of AST-classes for MQL
 *
 * Ulrik Petersen
 * Created: 11/22-2002
 * Last update: 11/22-2002
 *
 */
/************************************************************************
 *
 *   Emdros - the database engine for analyzed or annotated text
 *   Copyright (C) 2002  Ulrik Petersen
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, license version 2.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *
 *   Other licensing forms
 *   =====================
 *
 *   If you wish to negotiate commercial licensing, please contact
 *   Ulrik Petersen at ulrikp[at]users.sourceforge.net.
 *
 *   Licensing can also be negotiated if your organization is an
 *   educational, non-profit, charity, missionary or similar
 *   organization.
 *
 **************************************************************************/




#ifndef MQL_MONADS_STATEMENTS__H__
#define MQL_MONADS_STATEMENTS__H__

#include <mql_types.h>
#include <mql_execution_environment.h>

typedef enum { kSOUnion, kSOIntersect, kSODifference, kSONone } eSetOperator;

class CMonadSetChainElement {
private:
  CMonadSetElement *m_set;
  CMonadSetChainElement *m_next;
  eSetOperator m_operator;
public:
  CMonadSetChainElement(CMonadSetElement *myset, 
			eSetOperator op, 
			CMonadSetChainElement *next);
  virtual ~CMonadSetChainElement();
  CMonadSetElement *get_set(void) { return m_set; };
  eSetOperator get_operator(void) const { return m_operator; };
  CMonadSetChainElement *get_Next(void) { return m_next; };
  void set_Next(CMonadSetChainElement *next) { m_next = next; };
  void WeedMonotonic(CMQL_execution_environment *pEE, bool& bResult);
  void WeedPositive(CMQL_execution_environment *pEE, bool& bResult);
  void Exec(CSet_of_monad_ms& monads);
};

// Class for monad set calculation
class CMonadSetCalculationStatement : public CStatement {
private:
  CMonadSetChainElement *m_chain;
public:
  CMonadSetCalculationStatement(CMQL_execution_environment *pEE, CMonadSetChainElement *chain);
  virtual ~CMonadSetCalculationStatement();
  virtual void Weed(bool& bResult);
  virtual bool Exec();
};

// Base class for Insert and Delete monads
class CMonadsStatement : public CStatement {
protected:
  CMonadSetElement* m_AST_monads;
  CSet_of_monad_ms m_monads;
  CSet_of_monad_ms m_segment;
public:
  CMonadsStatement(CMQL_execution_environment *pEE, 
		   CMonadSetElement* monads) : CStatement(pEE), m_AST_monads(monads) {};
  virtual ~CMonadsStatement();
  virtual void Weed(bool& bResult);
  virtual bool Symbol(bool& bResult);
};

// INSERT MONADS
class CInsertMonadsStatement : public CMonadsStatement {
public:
  CInsertMonadsStatement(CMQL_execution_environment *pEE, 
			 CMonadSetElement* monads) : CMonadsStatement(pEE, monads) {};
  virtual ~CInsertMonadsStatement() {};
  virtual void Weed(bool& bResult);
  virtual bool Symbol(bool& bResult);
  virtual bool Exec();
};

// DELETE MONADS
class CDeleteMonadsStatement : public CMonadsStatement {
public:
  CDeleteMonadsStatement(CMQL_execution_environment *pEE, 
			 CMonadSetElement* monads) : CMonadsStatement(pEE, monads) {};
  virtual ~CDeleteMonadsStatement() {};
  virtual void Weed(bool& bResult);
  virtual bool Symbol(bool& bResult);
  virtual bool Exec();
};


// GET MONADS
class CGetMonadsStatement : public CStatement {
  CID_D* m_id_ds;
  std::string *m_object_type_name;
  id_d_t m_object_type_id;
public:
  CGetMonadsStatement(CMQL_execution_environment *pEE, 
		      CID_D *pID_Ds, 
		      std::string* object_type_name);
  virtual ~CGetMonadsStatement();
  virtual bool Exec();
};





#endif /* MQL_MONADS_STATEMENTS__H__ */
