/*
 * mql_object.h
 *
 * MQL representation of an object while processing the query
 *
 * Ulrik Petersen
 * Created: 4/2-2001
 * Last update: 1/18-2003
 *
 */
/************************************************************************
 *
 *   Emdros - the database engine for analyzed or annotated text
 *   Copyright (C) 2001-2003  Ulrik Petersen
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, license version 2.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *
 *   Other licensing forms
 *   =====================
 *
 *   If you wish to negotiate commercial licensing, please contact
 *   Ulrik Petersen at ulrikp[at]users.sourceforge.net.
 *
 *   Licensing can also be negotiated if your organization is an
 *   educational, non-profit, charity, missionary or similar
 *   organization.
 *
 **************************************************************************/




#ifndef MQL_OBJECT__H__
#define MQL_OBJECT__H__

#include <emdf.h>
#include <emdf_value.h>
#include <string>
#include <map>
#include <list>
#include <vector>
#include <emdfdb.h>
#include <inst_object.h>

class CMQLObject {
private:
  std::vector<CEMdFValue*> m_feature_value_vec;
  std::list<feature_info> m_feature_infos;
  std::vector<std::string> *m_feature_name_vec;
public:
  CMQLObject();
  ~CMQLObject();
  void add_feature(const feature_info& fi);
  void retrieve_feature_values(EMdFDB* pDB, const std::string& object_type_name, id_d_t object_id_d);
  void retrieve_feature_values(CInstObject *pOther);
  const CEMdFValue* get_feature_value(short int index) const { return m_feature_value_vec[index]; };
  short int get_feature_index(const std::string& feature_name) const;
  const std::vector<std::string>& get_feature_names(void);
  const std::list<feature_info>& get_feature_infos(void) { return m_feature_infos; };
};

#endif /* MQL_OBJECT__H__ */
