/*
 * mql_segment_statements.h
 *
 * Definitions of AST-classes for MQL
 *
 * Ulrik Petersen
 * Created: 11/22-2002
 * Last update: 11/22-2002
 *
 */
/************************************************************************
 *
 *   Emdros - the database engine for analyzed or annotated text
 *   Copyright (C) 2002  Ulrik Petersen
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, license version 2.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *
 *   Other licensing forms
 *   =====================
 *
 *   If you wish to negotiate commercial licensing, please contact
 *   Ulrik Petersen at ulrikp[at]users.sourceforge.net.
 *
 *   Licensing can also be negotiated if your organization is an
 *   educational, non-profit, charity, missionary or similar
 *   organization.
 *
 **************************************************************************/




#ifndef MQL_SEGMENT_STATEMENTS__H__
#define MQL_SEGMENT_STATEMENTS__H__

#include <mql_types.h>
#include <mql_execution_environment.h>

// Base class for the segment manipulation statements
class CSegmentStatement : public CStatement {
protected:
  std::string* m_segment_name;
public:
  CSegmentStatement(CMQL_execution_environment *pEE, std::string* segment_name) : CStatement(pEE), m_segment_name(segment_name) {};
  virtual ~CSegmentStatement();
};

// CREATE SEGMENT
class CCreateSegmentStatement : public CSegmentStatement {
private:
  CMonadSetElement* m_segment_range;
public:
  CCreateSegmentStatement(CMQL_execution_environment *pEE, 
			  std::string* segment_name, 
			  CMonadSetElement* segment_range);
  virtual ~CCreateSegmentStatement();
  virtual void Weed(bool& bResult);
  virtual bool Symbol(bool& bResult);
  virtual bool Exec();
};

// DROP SEGMENT
class CDropSegmentStatement : public CSegmentStatement {
public:
  CDropSegmentStatement(CMQL_execution_environment *pEE, 
			std::string* segment_name);
  virtual ~CDropSegmentStatement();
  virtual bool Symbol(bool& bResult);
  virtual bool Exec();
};




#endif /* MQL_SEGMENT_STATEMENTS__H__ */
