/*
 * mql_select_statements.h
 *
 * Definitions of AST-classes for MQL
 *
 * Ulrik Petersen
 * Created: 11/22-2002
 * Last update: 11/22-2002
 *
 */
/************************************************************************
 *
 *   Emdros - the database engine for analyzed or annotated text
 *   Copyright (C) 2002  Ulrik Petersen
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, license version 2.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *
 *   Other licensing forms
 *   =====================
 *
 *   If you wish to negotiate commercial licensing, please contact
 *   Ulrik Petersen at ulrikp[at]users.sourceforge.net.
 *
 *   Licensing can also be negotiated if your organization is an
 *   educational, non-profit, charity, missionary or similar
 *   organization.
 *
 **************************************************************************/




#ifndef MQL_SELECT_STATEMENTS__H__
#define MQL_SELECT_STATEMENTS__H__

#include <mql_types.h>
#include <mql_enumeration_statements.h>
#include <mql_object_type_statements.h>
#include <mql_execution_environment.h>

// SELECT (focus|all|) OBJECTS
class CSelectStatement : public CStatement, public CQueryBase {
public:
  CSelectStatement(CMQL_execution_environment *pEE, 
		   eFocusSpec focus, 
		   CMonadSetElement* in_spec, 
		   std::string* using_segment,
		   sheaf_return_type_t sheaf_return_type,
		   CTopograph* query);
  virtual ~CSelectStatement();
  virtual void Weed(bool& bResult);
  virtual bool Symbol(bool& bResult);
  virtual bool Type(bool& bResult);
  virtual bool Monads(bool& bResult);
  virtual bool Exec();
};

// SELECT OBJECTS AT
class CSelectObjectsAtStatement : public CStatement {
private:
  monad_m m_at_monad;
  std::string* m_object_type_name;
  id_d_t m_object_type_id;
public:
  CSelectObjectsAtStatement(CMQL_execution_environment *pEE, 
			    monad_m at_monad, std::string* object_type_name);
  virtual ~CSelectObjectsAtStatement();
  virtual bool Symbol(bool& bResult);
  virtual bool Exec();
};

// SELECT OBJECTS HAVING MONADS IN
class CSelectObjectsHavingMonadsInStatement : public CStatement {
private:
  CMonadSetElement* m_AST_monads;
  CSet_of_monad_ms m_monads;
  std::string* m_object_type_name;
  id_d_t m_object_type_id;
public:
  CSelectObjectsHavingMonadsInStatement(CMQL_execution_environment *pEE, 
					CMonadSetElement* monads, 
					std::string* object_type_name);
  virtual ~CSelectObjectsHavingMonadsInStatement();
  virtual void Weed(bool& bResult);
  virtual bool Symbol(bool& bResult);
  virtual bool Monads(bool& bResult);
  virtual bool Exec();
private:
  bool ExecOneObjectType(const std::string& object_type_name);
};

// SELECT OBJECT TYPES
class CSelectObjectTypesStatement : public CStatement {
public:
  CSelectObjectTypesStatement(CMQL_execution_environment *pEE);
  virtual ~CSelectObjectTypesStatement();
  virtual bool Exec();
};

// SELECT FEATURES
class CSelectFeaturesStatement : public CObjectTypeStatement {
public:
  CSelectFeaturesStatement(CMQL_execution_environment *pEE, 
			   std::string* object_type_name) : CObjectTypeStatement(pEE, object_type_name) {};
  virtual ~CSelectFeaturesStatement() {}
  virtual bool Symbol(bool& bResult);
  virtual bool Exec();
};

// SELECT ENUMERATIONS
class CSelectEnumerationsStatement : public CStatement {
public:
  CSelectEnumerationsStatement(CMQL_execution_environment *pEE);
  virtual ~CSelectEnumerationsStatement();
  virtual bool Exec();
};

// SELECT ENUMERATION CONSTANTS
class CSelectEnumerationConstantsStatement : public CEnumerationStatement {
public:
  CSelectEnumerationConstantsStatement(CMQL_execution_environment *pEE, 
				       std::string* enumeration_name) : CEnumerationStatement(pEE, enumeration_name) {};
  virtual ~CSelectEnumerationConstantsStatement() {};
  virtual bool Symbol(bool& bResult);
  virtual bool Exec();
};

// SELECT OBJECT TYPES USING ENUMERATION
class CSelectObjectTypesUsingEnumerationStatement : public CEnumerationStatement {
public:
  CSelectObjectTypesUsingEnumerationStatement(CMQL_execution_environment *pEE, 
					      std::string* enumeration_name) : CEnumerationStatement(pEE, enumeration_name) {};
  virtual ~CSelectObjectTypesUsingEnumerationStatement() {};
  virtual bool Symbol(bool& bResult);
  virtual bool Exec();
};

// SELECT SEGMENTS
class CSelectSegmentsStatement : public CStatement {
public:
  CSelectSegmentsStatement(CMQL_execution_environment *pEE) : CStatement(pEE) {};
  virtual ~CSelectSegmentsStatement() {};
  virtual bool Exec();
};

// SELECT MAX_M
class CSelectMaxMStatement : public CStatement {
public:
  CSelectMaxMStatement(CMQL_execution_environment *pEE) : CStatement(pEE) {};
  virtual ~CSelectMaxMStatement() {};
  virtual bool Exec();
};

// SELECT MIN_M
class CSelectMinMStatement : public CStatement {
public:
  CSelectMinMStatement(CMQL_execution_environment *pEE) : CStatement(pEE) {};
  virtual ~CSelectMinMStatement() {};
  virtual bool Exec();
};




#endif /* MQL_SELECT_STATEMENTS__H__ */
